# Copyright 2023 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Try to find CoDiPack headers
# ----------------------------------------------------------------------------   
# output:
#    CODI_FOUND             : whether the packae was found
#    CODI_INCLUDE_DIRS      : where the headers are [cached]
#    CODI_VERSION           : version number
# ----------------------------------------------------------------------------
# autodetection:
#    use "CMAKE_PREFIX_PATH=/path/to/package/"
#    or  "CMAKE_INCLUDE_PATH=/path/to/package/include"
#    or  "INCLUDE=/path/to/package/include"
# ----------------------------------------------------------------------------   

# Check version
macro(_check_version)
    file(READ "${CODI_INCLUDE_DIRS}/codi.hpp" _version_header)

    string(REGEX MATCH "define[ \t]+CODI_MAJOR_VERSION[ \t]+([0-9]+)" major_version_match "${_version_header}")
    set(CODI_MAJOR_VERSION "${CMAKE_MATCH_1}")                        
    string(REGEX MATCH "define[ \t]+CODI_MINOR_VERSION[ \t]+([0-9]+)" minor_version_match "${_version_header}")
    set(CODI_MINOR_VERSION "${CMAKE_MATCH_1}")                        
    string(REGEX MATCH "define[ \t]+CODI_BUILD_VERSION[ \t]+([0-9]+)" build_version_match "${_version_header}")
    set(CODI_BUILD_VERSION "${CMAKE_MATCH_1}")

    set(CODI_VERSION ${CODI_MAJOR_VERSION}.${CODI_MINOR_VERSION}.${CODI_BUILD_VERSION})

    if(${CODI_VERSION} VERSION_LESS ${CODI_FIND_VERSION})
        message(FATAL_ERROR "CoDiPack version ${CODI_VERSION} found in ${CODI_INCLUDE_DIRS}, but at least version ${CODI_FIND_VERSION} is required!")
    endif()
endmacro()

# Set a dummy version if not provided
if(NOT CODI_FIND_VERSION)
    if(NOT CODI_FIND_VERSION_MAJOR)
        set(CODI_FIND_VERSION_MAJOR 2)
    endif()
    if(NOT CODI_FIND_VERSION_MINOR)
        set(CODI_FIND_VERSION_MINOR 0)
    endif()
    if(NOT CODI_FIND_VERSION_BUILD)
        set(CODI_FIND_VERSION_BUILD 2)
    endif()

    set(CODI_FIND_VERSION "${CODI_FIND_VERSION_MAJOR}.${CODI_FIND_VERSION_MINOR}.${CODI_FIND_VERSION_BUILD}")
endif()

# Find the header and check the version
find_path(CODI_INCLUDE_DIRS "codi.hpp")
if (CODI_INCLUDE_DIRS)
    _check_version()
else()
    message(FATAL_ERROR "CoDiPack headers not found! Define the path in the INCLUDE environment variable.")
endif()

# handle the QUIETLY and REQUIRED arguments and set EIGEN_FOUND to TRUE
# if all listed variables are TRUE
include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(CODI
                                  FOUND_VAR CODI_FOUND
                                  REQUIRED_VARS CODI_INCLUDE_DIRS
                                  VERSION_VAR CODI_VERSION)
