/*
 * Copyright 2023 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SDPM_H
#define SDPM_H

#if defined(WIN32)
#ifdef sdpm_EXPORTS
#define SDPM_API __declspec(dllexport)
#else
#define SDPM_API __declspec(dllimport)
#endif
#else
#define SDPM_API
#endif

#ifdef _MSC_VER
#pragma warning(disable : 4251) // DLL/templates non exportes
#endif

#include "sdpm_def.h"
#include <Eigen/Dense>
#include <complex>

#ifndef SWIG
#ifndef USE_CODI
using sdpmDouble = double;
#else // USE_CODI
#include "codi.hpp"
using sdpmDouble = codi::RealReverse;
using sdpmTape = sdpmDouble::Tape;
#endif // USE_CODI

using sdpmComplex = std::complex<sdpmDouble>;
using sdpmMatrixXd = Eigen::Matrix<sdpmDouble, Eigen::Dynamic, Eigen::Dynamic>;
using sdpmVectorXd = Eigen::Matrix<sdpmDouble, Eigen::Dynamic, 1>;
using sdpmMatrix3d = Eigen::Matrix<sdpmDouble, 3, 3>;
using sdpmVector3d = Eigen::Matrix<sdpmDouble, 3, 1>;
using sdpmMatrixXcd = Eigen::Matrix<sdpmComplex, Eigen::Dynamic, Eigen::Dynamic>;
using sdpmVectorXcd = Eigen::Matrix<sdpmComplex, Eigen::Dynamic, 1>;
using sdpmVector3cd = Eigen::Matrix<sdpmComplex, 3, 1>;
#endif // SWIG

namespace sdpm
{
// Utilities
class sdpmObject;
class Timers;
class UnitTest;
// Mesh management
class Mesh;
class Node;
class Element;
class Line2;
class Quad4;
class Tag;
class Group;
class Result;
class GmshImport;
class GmshExport;
// Panel method
class Problem;
class Body;
class Wake;
class Motion;
class Edge;
class Builder;
class Solver;
class SolverTransonic;
class Adjoint;
class Gradient;
} // namespace sdpm

#endif // SDPM_H
