/*
 * Copyright 2023 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SDPMADJOINT_H
#define SDPMADJOINT_H

#include "sdpm.h"

#ifdef USE_CODI
#include "sdpmObject.h"
#include <map>
#include <string>
#include <vector>

namespace sdpm
{

/**
 * @brief Source-doublet panel adjoint interface
 * @authors Adrien Crovato
 */
class SDPM_API Adjoint : public sdpmObject
{
    Solver &_sol; ///< SDPM solver
    size_t _nm;   ///< number of modes
    size_t _nf;   ///< number of frequencies
    // Inputs
    sdpmDouble &_aoa;                                      ///< angle of attack
    std::map<std::string, std::vector<sdpmVector3d *>> _x; ///< body node coordinates
    std::vector<sdpmDouble> &_omega;                       /// frequencies
    // Outputs
    sdpmDouble &_cl;                              ///< steady lift coefficient
    sdpmDouble &_cd;                              ///< steady drag coefficient
    std::vector<std::vector<sdpmDouble>> &_cl1Re; ///< real part of unsteady first harmonic lift coefficient
    std::vector<std::vector<sdpmDouble>> &_cl1Im; ///< imaginary part of unsteady first harmonic lift coefficient
    std::vector<std::vector<sdpmDouble>> &_cd1Re; ///< real part of unsteady first harmonic drag coefficient
    std::vector<std::vector<sdpmDouble>> &_cd1Im; ///< imaginary part of unsteady first harmonic drag coefficient

public:
    Adjoint(Solver &sol);
    ~Adjoint();

    void solve();
    std::map<std::string, std::vector<double>> compute(std::string const &dOutName, std::vector<double> const &dOut, std::vector<std::string> const &dInNames);

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace sdpm

#endif // USE_CODI
#endif // SDPMADJOINT_H
