/*
 * Copyright 2023 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SDPMBUILDER_H
#define SDPMBUILDER_H

#include "sdpm.h"
#include "sdpmObject.h"
#include <vector>

namespace sdpm
{

/**
 * @brief Aerodynamic Influence Coefficients matrices builder
 * @authors Adrien Crovato
 */
class SDPM_API Builder
{
public:
    static sdpmMatrix3d computeTransfoMat(bool symy, Element const *ej);
    static sdpmMatrixXd gatherCoords(bool symy, Element const *ej);
    static sdpmVector3d computeDistVec(bool symy, Element const *ei, Element const *ej);
    static std::vector<sdpmDouble> computeAIC(bool symy, Element const *ei, Element const *ej, sdpmMatrixXd const &trsfCoordSrc, sdpmMatrix3d const &g2l);
};

} // namespace sdpm
#endif // SDPMBUILDER_H
