/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SDPMGMSHIMPORT_H
#define SDPMGMSHIMPORT_H

#include "sdpm.h"
#include "sdpmObject.h"
#include "sdpmMesh.h"
#include <vector>
#include <map>

namespace sdpm
{

/**
 * @brief Read Gmsh mesh file and load into data structure
 * @authors Adrien Crovato
 * @todo replace by Gmsh API?
 */
class SDPM_API GmshImport : public sdpmObject
{
    Mesh &_msh;                        ///< mesh data structure
    bool _verb;                        ///< verbosity flag
    std::map<size_t, Node *> _nodeMap; ///< id-node map
    std::map<size_t, Tag *> _tagMap;   ///< id-node map

    int searchCode(char const *code, char const *line) const;
    void readMeshFormat(FILE *file);
    void readPhysicalTags(FILE *file);
    void readNodes(FILE *file);
    void readElements(FILE *file);
    void readElNods(FILE *file, int n, std::vector<Node *> &nods);
    Tag *findTag(FILE *file);

public:
    GmshImport(Mesh &msh, bool v = false);
    ~GmshImport();

    void load(std::string const &fname);
    void write(std::ostream &out) const;
};

} // namespace sdpm

#endif // SDPMGMSHIMPORT_H
