/*
 * Copyright 2023 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SDPMMESH_H
#define SDPMMESH_H

#include "sdpm.h"
#include "sdpmObject.h"
#include "sdpmNode.h"
#include "sdpmElement.h"
#include "sdpmTag.h"
#include <string>
#include <vector>
#include <map>

namespace sdpm
{

/**
 * @brief Mesh data structure
 * @authors Adrien Crovato
 */
class SDPM_API Mesh : public sdpmObject
{
    std::string _name;                  ///< mesh name
    std::vector<Node *> _nodes;         ///< nodes
    std::vector<Element *> _elements;   ///< elements
    std::map<std::string, Tag *> _tags; ///< named (physical) tags
public:
    Mesh();
    ~Mesh();

    // Getters
    std::string const &getName() const { return _name; }
    std::vector<Node *> const &getNodes() const { return _nodes; }
    std::vector<Element *> const &getElements() const { return _elements; }
    std::map<std::string, Tag *> const &getTags() const { return _tags; }
#ifndef SWIG
    // Setters
    void setName(std::string const &n) { _name = n; }
    void addNode(Node *n) { _nodes.push_back(n); }
    void addElement(Element *e) { _elements.push_back(e); }
    void addTag(std::string const &n, Tag *t) { _tags[n] = t; }
    void reserveNodes(size_t n) { _nodes.reserve(n); }
    void reserveElements(size_t n) { _elements.reserve(n); }
#endif

    void clear();
#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace sdpm

#endif // SDPMMESH_H
