/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SDPMOBJECT_H
#define SDPMOBJECT_H

#include "sdpm.h"
#include <ostream>

namespace sdpm
{

/**
 * @brief Base class of all virtual objects to easiliy print in Python (using __str__())
 * @authors Adrien Crovato
 */

class SDPM_API sdpmObject
{
public:
    sdpmObject() = default;
    virtual ~sdpmObject() {}
    sdpmObject(const sdpmObject &) = delete;
    sdpmObject &operator=(const sdpmObject &) = delete;

#ifndef SWIG
    friend SDPM_API std::ostream &operator<<(std::ostream &out, sdpmObject const &obj)
    {
        obj.write(out);
        return out;
    }
    virtual void write(std::ostream &out) const {}
#endif
};

} // namespace sdpm

#endif // SDPMOBJECT_H
