/*
 * Copyright 2023 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SDPMRESULTS_H
#define SDPMRESULTS_H

#include "sdpm.h"
#include "sdpmObject.h"
#include <string>
#include <vector>
#include <map>

namespace sdpm
{

/**
 * @brief Results data structure
 * @authors Adrien Crovato
 */
class SDPM_API Results : public sdpmObject
{
public:
    int nt;
    double time;
    std::map<std::string, std::vector<sdpmDouble> *> scalarsAtNodes;
    std::map<std::string, std::vector<sdpmDouble> *> scalarsAtElems;
    std::map<std::string, std::vector<sdpmVector3d> *> vectorsAtNodes;
    std::map<std::string, std::vector<sdpmVector3d> *> vectorsAtElems;

    Results() : sdpmObject(), nt(0), time(0.0) {}

    virtual void write(std::ostream &out) const override;
};

} // namespace sdpm

#endif // SDPMRESULTS_H
