/*
 * Copyright 2023 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "sdpmUnitTest.h"
#include "sdpmMesh.h"
#include "sdpmGmshExport.h"
#include "sdpmResults.h"

using namespace sdpm;

/**
 * @brief Write mesh and results to disk
 */
void UnitTest::gmshIO(Mesh &mesh, GmshExport &writer)
{
    // Get nodes and elements
    std::vector<Node *> nods = mesh.getNodes();
    std::vector<Element *> elems = mesh.getElements();

    // Variables to be written
    std::vector<sdpmDouble> x(nods.size(), 0.), type(elems.size(), 0.);                                     // x-position of node, element type
    std::vector<sdpmVector3d> v(nods.size(), sdpmVector3d::Zero()), cg(elems.size(), sdpmVector3d::Zero()); // x,y,z-position of node and element cg

    // Temporary results container
    Results res;
    res.scalarsAtNodes["x"] = &x;
    res.vectorsAtNodes["v"] = &v;
    res.scalarsAtElems["type"] = &type;
    res.vectorsAtElems["cg"] = &cg;

    // Fill the variables and write them to disk
    for (size_t i = 0; i < nods.size(); ++i)
    {
        x[i] = nods[i]->getCoords()(0);
        v[i] = nods[i]->getCoords();
    }
    for (size_t i = 0; i < elems.size(); ++i)
    {
        type[i] = static_cast<double>(elems[i]->getType());
        cg[i] = elems[i]->getCg();
    }
    writer.save(res, "_new");
}
